package ch.polgrabia.demos.client;

import io.vertx.core.Handler;
import io.vertx.core.http.WebSocket;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;

public class ClientPeriodicStdinScanner implements Handler<Long> {
    private static final Logger logger = LoggerFactory.getLogger(ClientPeriodicStdinScanner.class);
    private final InputStream inputStream;
    private ByteArrayOutputStream clientTextInput = new ByteArrayOutputStream();
    private final WebSocket webSocket;

    ClientPeriodicStdinScanner(WebSocket webSocket, InputStream inputStream) {
        this.webSocket = webSocket;
        this.inputStream = inputStream;
    }

    private static int newLineIndex(byte[] clientPayload) {
        for (int i = 0; i < clientPayload.length; i++) {
            if (clientPayload[i] == "\n".getBytes(StandardCharsets.UTF_8)[0]) {
                return i;
            }
        }
        return -1;
    }

    @Override
    public void handle(Long event) {
        {
            try {
                int available = inputStream.available();
                if (available > 0) {
                    byte[] clientPayload = inputStream.readNBytes(available);
                    this.clientTextInput.write(clientPayload);
                    int idx = newLineIndex(clientPayload);
                    if (idx > 0) {
                        String payload = clientTextInput.toString(StandardCharsets.UTF_8);
                        // TODO here don't assume we deal with linux eol
                        int nidx = payload.indexOf('\n');
                        String textMsgToBeSent = payload.substring(0, nidx);
                        webSocket.writeTextMessage(textMsgToBeSent);
                        clientTextInput = new ByteArrayOutputStream();
                        clientTextInput.write(payload.substring(idx).getBytes(StandardCharsets.UTF_8));
                    }
                }
            } catch (IOException e) {
                logger.error("Failed to read client input", e);
            }
        }
    }
}
